@@ -50,6 +50,7 @@
 #include "kopenwithdialog.h"
 #include "krecentdocument.h"
 #include "kdesktopfileactions.h"
+#include "executablefileopendialog_p.h"
 #include <kio/desktopexecparser.h>
 
 #include <kurlauthorized.h>
@@ -176,7 +177,8 @@ bool KRun::runUrl(const QUrl &u, const QString &_mimetype, QWidget *window, bool
     } else if (isExecutableFile(u, _mimetype)) {
         if (u.isLocalFile() && runExecutables) {
             if (KAuthorized::authorize("shell_access")) {
-                return (KRun::runCommand(KShell::quoteArg(u.toLocalFile()), QString(), QString(), window, asn, u.adjusted(QUrl::RemoveFilename).path())); // just execute the url as a command
+                return (KRun::runCommand(KShell::quoteArg(u.toLocalFile()), QString(), QString(),
+                            window, asn, u.adjusted(QUrl::RemoveFilename).toLocalFile())); // just execute the url as a command
                 // ## TODO implement deleting the file if tempFile==true
             } else {
                 noAuth = true;
@@ -256,7 +258,7 @@ bool KRun::displayOpenWithDialog(const QList<QUrl> &lst, QWidget *window, bool t
     return false;
 }
 
-#ifndef KDE_NO_DEPRECATED
+#ifndef KIOWIDGETS_NO_DEPRECATED
 void KRun::shellQuote(QString &_str)
 {
     // Credits to Walter, says Bernd G. :)
@@ -276,7 +278,7 @@ QStringList KRun::processDesktopExec(const KService &_service, const QList<QUrl>
     return parser.resultingArguments();
 }
 
-#ifndef KDE_NO_DEPRECATED
+#ifndef KIOWIDGETS_NO_DEPRECATED
 QString KRun::binaryName(const QString &execLine, bool removePath)
 {
     return removePath ? KIO::DesktopExecParser::executableName(execLine) : KIO::DesktopExecParser::executablePath(execLine);
@@ -426,8 +428,13 @@ static bool runTempService(const KService &_service, const QList<QUrl> &_urls, Q
     }
     //qDebug() << "runTempService: KProcess args=" << args;
 
+    QString path(_service.path());
+    if (path.isEmpty() && !_urls.isEmpty() && _urls.first().isLocalFile()) {
+        path = _urls.first().adjusted(QUrl::RemoveFilename).toLocalFile();
+    }
+
     return runCommandInternal(args.join(" "), &_service, KIO::DesktopExecParser::executablePath(_service.exec()),
-                              _service.name(), _service.icon(), window, asn, _service.path());
+                              _service.name(), _service.icon(), window, asn, path);
 }
 
 // WARNING: don't call this from DesktopExecParser, since klauncher uses that too...
@@ -823,6 +830,7 @@ void KRun::KRunPrivate::init(const QUrl &url, QWidget *window,
     // Start the timer. This means we will return to the event
     // loop and do initialization afterwards.
     // Reason: We must complete the constructor before we do anything else.
+    m_bCheckPrompt = false;
     m_bInit = true;
     q->connect(m_timer, SIGNAL(timeout()), q, SLOT(slotTimeout()));
     startTimer();
@@ -963,6 +971,57 @@ bool KRun::KRunPrivate::runExecutable(const QString &_exec)
     return false;
 }
 
+void KRun::KRunPrivate::showPrompt()
+{
+    ExecutableFileOpenDialog *dialog = new ExecutableFileOpenDialog(q->window());
+    dialog->setAttribute(Qt::WA_DeleteOnClose);
+    connect(dialog, &ExecutableFileOpenDialog::finished, q, [this, dialog](int result){
+        onDialogFinished(result, dialog->isDontAskAgainChecked());
+        });
+    dialog->show();
+}
+
+bool KRun::KRunPrivate::isPromptNeeded()
+{
+    const QMimeDatabase db;
+    const QMimeType mime = db.mimeTypeForUrl(m_strURL);
+
+    const bool isFileExecutable = (isExecutableFile(m_strURL, mime.name()) ||
+                                   mime.inherits("application/x-desktop"));
+    const bool isTextFile = mime.inherits("text/plain");
+
+    if (isFileExecutable && isTextFile) {
+        KConfigGroup cfgGroup(KSharedConfig::openConfig("kiorc"), "Executable scripts");
+        const QString value = cfgGroup.readEntry("behaviourOnLaunch", "alwaysAsk");
+
+        if (value == "alwaysAsk") {
+            return true;
+        } else {
+            q->setRunExecutables(value == "execute");
+        }
+    }
+
+    return false;
+}
+
+void KRun::KRunPrivate::onDialogFinished(int result, bool isDontAskAgainSet)
+{
+    if (result == ExecutableFileOpenDialog::Rejected) {
+        m_bFinished = true;
+        m_bInit = false;
+        startTimer();
+        return;
+    }
+    q->setRunExecutables(result == ExecutableFileOpenDialog::ExecuteFile);
+
+    if (isDontAskAgainSet) {
+        QString output = result == ExecutableFileOpenDialog::OpenFile ? "open" : "execute";
+        KConfigGroup cfgGroup(KSharedConfig::openConfig("kiorc"), "Executable scripts");
+        cfgGroup.writeEntry("behaviourOnLaunch", output);
+    }
+    startTimer();
+}
+
 void KRun::scanFile()
 {
     //qDebug() << d->m_strURL;
@@ -1002,11 +1061,17 @@ void KRun::scanFile()
     //qDebug() << "Job" << job << "is about getting from" << d->m_strURL;
 }
 
-// When arriving in that method there are 5 possible states:
-// must_init, must_scan_file, found_dir, done+error or done+success.
+// When arriving in that method there are 6 possible states:
+// must_show_prompt, must_init, must_scan_file, found_dir, done+error or done+success.
 void KRun::slotTimeout()
 {
-    //qDebug() << this << " slotTimeout called";
+    if (d->m_bCheckPrompt) {
+        d->m_bCheckPrompt = false;
+        if (d->isPromptNeeded()) {
+            d->showPrompt();
+            return;
+        }
+    }
     if (d->m_bInit) {
         d->m_bInit = false;
         init();
@@ -1266,6 +1331,11 @@ void KRun::setSuggestedFileName(const QString &fileName)
     d->m_suggestedFileName = fileName;
 }
 
+void KRun::setShowScriptExecutionPrompt(bool showPrompt)
+{
+    d->m_bCheckPrompt = showPrompt;
+}
+
 QString KRun::suggestedFileName() const
 {
     return d->m_suggestedFileName;
@@ -1322,28 +1392,28 @@ KIO::Job *KRun::job()
     return d->m_job;
 }
 
-#ifndef KDE_NO_DEPRECATED
+#ifndef KIOWIDGETS_NO_DEPRECATED
 QTimer &KRun::timer()
 {
     return *d->m_timer;
 }
 #endif
 
-#ifndef KDE_NO_DEPRECATED
+#ifndef KIOWIDGETS_NO_DEPRECATED
 void KRun::setDoScanFile(bool scanFile)
 {
     d->m_bScanFile = scanFile;
 }
 #endif
 
-#ifndef KDE_NO_DEPRECATED
+#ifndef KIOWIDGETS_NO_DEPRECATED
 bool KRun::doScanFile() const
 {
     return d->m_bScanFile;
 }
 #endif
 
-#ifndef KDE_NO_DEPRECATED
+#ifndef KIOWIDGETS_NO_DEPRECATED
 void KRun::setIsDirecory(bool isDirectory)
 {
     d->m_bIsDirectory = isDirectory;
@@ -1355,14 +1425,14 @@ bool KRun::isDirectory() const
     return d->m_bIsDirectory;
 }
 
-#ifndef KDE_NO_DEPRECATED
+#ifndef KIOWIDGETS_NO_DEPRECATED
 void KRun::setInitializeNextAction(bool initialize)
 {
     d->m_bInit = initialize;
 }
 #endif
 
-#ifndef KDE_NO_DEPRECATED
+#ifndef KIOWIDGETS_NO_DEPRECATED
 bool KRun::initializeNextAction() const
 {
     return d->m_bInit;
@@ -1426,7 +1496,7 @@ qint64 KProcessRunner::pid() const
 void KProcessRunner::terminateStartupNotification()
 {
 #if HAVE_X11
-    if (!id.none()) {
+    if (!id.isNull()) {
         KStartupInfoData data;
         data.addPid(m_pid); // announce this pid for the startup notification has finished
         data.setHostname();
